#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from neutron_lbaas.services.loadbalancer import constants

ROUND_ROBIN = 'rr'
LEAST_CONNECTIONS = 'lc'
SOURCE_IP = 'sh'
PERS_SOURCE_IP = 'src'
PERS_HTTP_COOKIE = 'cookie'
PERS_ACT_COOKIE = 'active-cookie'

LB_METHODS = {
    constants.LB_METHOD_ROUND_ROBIN: ROUND_ROBIN,
    constants.LB_METHOD_LEAST_CONNECTIONS: LEAST_CONNECTIONS,
    constants.LB_METHOD_SOURCE_IP: SOURCE_IP,
}

PERSIST_OPTS = {
    constants.SESSION_PERSISTENCE_SOURCE_IP: PERS_SOURCE_IP,
    constants.SESSION_PERSISTENCE_HTTP_COOKIE: PERS_HTTP_COOKIE,
    constants.SESSION_PERSISTENCE_APP_COOKIE: PERS_ACT_COOKIE,
}

VS_ID = [
    "vs",
    "vsport",
    "vsprot",
]

RS_ID = [
    "rs",
    "rsport",
]

CHECKER_OPTS = [
    'retryinterval',
    'timeout',
    'retrycount',
]



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from oslo.config import cfg
from oslo_log import log

from neutron_lbaas.services.loadbalancer import constants as lb_constants

import client
import constants as kemp_consts

LOG = log.getLogger(__name__)


class ManagerBase(object):
    def __init__(self, driver, manager):
        self.driver = driver.openstack_driver
        self.client = driver.client
        self.api_helper = driver.api_helper
        self.manager = manager


class LoadBalancerManager(ManagerBase):

    def create(self, context, lb):
        LOG.debug("KEMP driver create lb: %s" % repr(lb))
        self.manager.successful_completion(context, lb)

    def update(self, context, old_lb, lb):
        LOG.debug("KEMP driver update lb: %s" % repr(lb))
        self.driver.plugin.activate_linked_entities(context, lb)

    def delete(self, context, lb):
        LOG.debug("KEMP driver delete lb: %s" % repr(lb))
        self.driver.plugin.db.delete_loadbalancer(context, lb.id)
        self.manager.successful_completion(context, lb, delete=True)

    def refresh(self, context, lb):
        LOG.debug("KEMP driver refresh lb: %s" % repr(lb))
        # TODO(smcgough) Compare to back end and fix inconsistencies.
        self.client.refresh_loadbalancer(lb)

    def stats(self, context, lb):
        LOG.debug("KEMP driver stats of lb: %s" % repr(lb))
        # TODO(smcgough) Return nothing for now.
        stats = {
            lb_constants.STATS_IN_BYTES: 0,
            lb_constants.STATS_OUT_BYTES: 0,
            lb_constants.STATS_ACTIVE_CONNECTIONS: 0,
            lb_constants.STATS_TOTAL_CONNECTIONS: 0,
        }
        return stats


class ListenerManager(ManagerBase):

    def create(self, context, listener):
        LOG.debug("KEMP driver create listener: %s" % repr(listener))
        self.manager.successful_completion(context, listener)

    def update(self, context, old_listener, listener):
        LOG.debug("KEMP driver update listener: %s" % repr(listener))
        old_prep_listener = self.api_helper.prepare_listener(context,
                                                             old_listener)
        prep_listener = self.api_helper.prepare_listener(context, listener)
        try:
            self.client.update_virtual_service(old_prep_listener, prep_listener)
            self.manager.successful_completion(context, listener)
        except client.KempClientRequestError() as ex:
            LOG.error("Error updating listener: %s", ex)
            self.manager.failed_completion(context, listener)

    def delete(self, context, listener):
        LOG.debug("KEMP driver delete listener: %s" % repr(listener))
        prep_listener = self.api_helper.prepare_listener(context, listener)
        try:
            self.client.delete_virtual_service(prep_listener)
            self.manager.successful_completion(context, listener, delete=True)
            self.driver.plugin.db.delete_listener(context, listener.id)
        except client.KempClientRequestError() as ex:
            LOG.error("Error deleting listener: %s", ex)
            self.manager.failed_completion(context, listener)


class PoolManager(ManagerBase):

    def create(self, context, pool):
        LOG.debug("KEMP driver create pool: %s" % repr(pool))
        prep_pool = self.api_helper.prepare_pool(context, pool)
        try:
            self.client.create_virtual_service(prep_pool)
            self.manager.successful_completion(context, pool)
        except client.KempClientRequestError() as ex:
            LOG.error("Error creating listener: %s", ex)
            self.manager.failed_completion(context, pool)

    def update(self, context, old_pool, pool):
        LOG.debug("KEMP driver update pool: %s" % repr(pool))
        if old_pool['lb_method'] != pool['lb_method']:
            prep_pool = self.api_helper.prepare_pool(context, pool)
            try:
                self.client.update_virtual_service(prep_pool)
                self.manager.successful_completion(context, pool)
            except client.KempClientRequestError() as ex:
                LOG.error("Error updating pool: %s", ex)
                self.manager.failed_completion(context, pool)

    def delete(self, context, pool):
        LOG.debug("KEMP driver delete pool: %s" % repr(pool))
        prep_pool = self.api_helper.prepare_pool(context, pool)
        try:
            self.client.delete_virtual_service(prep_pool)
            self.driver.plugin.db.delete_pool(context, pool.id)
            self.manager.successful_completion(context, pool, delete=True)
        except client.KempClientRequestError() as ex:
            LOG.error("Error deleting pool: %s", ex)
            self.manager.failed_completion(context, pool)


class MemberManager(ManagerBase):

    def create(self, context, member):
        LOG.debug("KEMP driver create member: %s" % repr(member))
        prep_member = self.api_helper.prepare_member(context, member)
        try:
            self.client.create_real_server(prep_member)
            self.manager.successful_completion(context, member)
        except client.KempClientRequestError() as ex:
            LOG.error("Error updating pool: %s", ex)
            self.manager.failed_completion(context, member)

    def update(self, context, old_member, member):
        LOG.debug("KEMP driver update member: %s" % repr(member))
        old_prep_member = self.api_helper.prepare_member(context, old_member)
        prep_member = self.api_helper.prepare_member(context, member)
        try:
            self.client.update_real_server(old_prep_member, prep_member)
            self.manager.successful_completion(context, member)
        except client.KempClientRequestError() as ex:
            LOG.error("Error updating member: %s", ex)
            self.driver.plugin.db.delete_member(context, member.id)
            self.manager.failed_completion(context, member)

    def delete(self, context, member):
        LOG.debug("KEMP driver delete member: %s" % repr(member))
        prep_member = self.api_helper.prepare_member(context, member)
        try:
            self.client.delete_real_server(prep_member)
            self.manager.successful_completion(context, member, delete=True)
        except client.KempClientRequestError() as ex:
            LOG.error("Error deleting member: %s", ex)
            self.driver.plugin.db.delete_member(context, member.id)
            self.manager.failed_completion(context, member)


class HealthMonitorManager(ManagerBase):

    def create(self, context, health_monitor):
        LOG.debug("KEMP driver create health_monitor['type']: %s"
                  % health_monitor['type'])
        check_params = self._get_health_check_params(health_monitor)
        if "http" in health_monitor['type']:
            params = self.api_helper.prepare_health_monitor(context, health_monitor)
            check_params.update(params)
        try:
            self.client.update_health_check(check_params)
            self.manager.successful_completion(context, health_monitor)
        except client.KempClientRequestError() as ex:
            LOG.error("Error creating health monitor: %s", ex)
            self.manager.failed_completion(context, health_monitor)

    def update(self, context, old_health_monitor, health_monitor):
        LOG.debug("KEMP driver update health_monitor: %s" %
                  repr(health_monitor))
        check_params = self._get_health_check_params(health_monitor)
        if "http" in health_monitor['type']:
            params = self.api_helper.prepare_health_monitor(context, health_monitor)
            check_params.update(params)
        try:
            self.client.update_health_check(check_params)
            self.manager.successful_completion(context, health_monitor)
        except client.KempClientRequestError() as ex:
            LOG.error("Error updating health monitor: %s", ex)
            self.manager.failed_completion(context, health_monitor)

    def delete(self, context, health_monitor):
        LOG.debug("KEMP driver delete health_monitor: %s" %
                  repr(health_monitor))
        check_params = self._get_health_check_params(health_monitor)
        if "http" in health_monitor['type']:
            params = self.api_helper.prepare_health_monitor(context, health_monitor)
            check_params.update(params)
        try:
            self.client.update_health_check(check_params)
            self.manager.successful_completion(context, health_monitor,
                                               delete=True)
        except client.KempClientRequestError() as ex:
            LOG.error("Error deleting health monitor: %s", ex)
            self.manager.failed_completion(context, health_monitor)
            self.driver.plugin.db.health_monitor(context, health_monitor.id)

    @staticmethod
    def _get_health_check_params(health_monitor):
        """Return health check parameters from a health monitor."""
        check_params = {
            'retryinterval': health_monitor['delay'],
            'timeout': health_monitor['timeout'],
            'retrycount': health_monitor['max_retries'],
        }
        return check_params


class MapNeutronToKemp(object):

    MAP_NEUTRON_MODEL_TO_VS = {
        'vip_address': 'vs',
        'protocol_port': 'port',
        'protocol': 'prot',
        'lb_method': 'schedule',
        'type': 'checktype',
        'http_method': 'checkuseget',
        'url_path': 'checkurl',
    }

    MAP_NEUTRON_MODEL_TO_RS = {
        'address': 'rs',
        'protocol_port': 'rsport',
        'weight': 'weight',
    }

    def __init__(self, plugin):
        self.plugin = plugin

    def prepare_load_balancer(self, context, loadbalancer):
        pass

    def prepare_listener(self, context, listener):
        pass

    def prepare_pool(self, context, pool):
        health_monitor = pool.healthmonitor or None
        listener = pool.listener
        loadbalancer = pool.root_loadbalancer
        vs_params = {}
        for model in [listener, loadbalancer, pool, health_monitor]:
            for key in dir(model):
                if not key.startswith('__') and not callable(getattr(model, key)):
                    try:
                        vs_key = self.MAP_NEUTRON_MODEL_TO_VS.get(key)
                        value = model.__dict__[key]
                    except KeyError as e:
                        vs_key = None
                        value = None
                    if key == "lb_algorithm":
                        vs_params[vs_key] = kemp_consts.LB_METHODS.get(value)
                    elif key == "http_method":
                        if model.http_method == "GET":
                            vs_params[vs_key] = 1
                        else:
                            vs_params[vs_key] = 0
                    elif vs_key is not None:
                        vs_params[vs_key] = value

        # Need to explicitly set vstype if port and
        # protocol do not meet default requirements
        if (("HTTP" in vs_params['prot']) and
                    vs_params['port'] != 80 and
                    vs_params['port'] != 443):
            vs_params['vstype'] = 'http'

        if pool.session_persistence is not None:
            for session_persist in lb_constants.SUPPORTED_SP_TYPES:
                if session_persist == pool.session_persistence.type:
                    persistence = kemp_consts.PERSIST_OPTS[session_persist]
                    vs_params['persist'] = persistence
                    if persistence == kemp_consts.PERS_ACT_COOKIE:
                        cookie = pool.session_persistence.cookie_name
                        vs_params['cookie'] = cookie
        return vs_params

    def prepare_member(self, context, member):
        rs_params = {}
        for key in dir(member):
            if not key.startswith('__') and not callable(getattr(member, key)):
                api_param = self.MAP_NEUTRON_MODEL_TO_RS.get(key)
                if api_param is not None:
                    rs_params[api_param] = member.key
        for key in dir(member.pool.listener):
            if key in kemp_consts.VS_ID:
                api_param = self.MAP_NEUTRON_MODEL_TO_VS.get(key)
                if api_param is not None:
                    rs_params[api_param] = member.key
        return rs_params

    def prepare_health_monitor(self, context, health_monitor):
        """Return prepare_pool to update the VS."""
        prep_hm = self.prepare_pool(context, health_monitor.pool)
        return prep_hm


class KempLoadMasterDriver(object):
    """KEMPtechnologies LBaaS driver."""

    def __init__(self, driver, config):
        try:
            self.address = config.lm_address
            self.username = config.lm_username
            self.password = config.lm_password
            self.check_interval = config.check_interval
            self.connect_timeout = config.connect_timeout
            self.retry_count = config.retry_count
            self.client = client.KempClient(self.address, self.username,
                                            self.password)
        except (cfg.NoSuchOptError, client.KempClientRequestError) as error:
            LOG.error(error)
        self.openstack_driver = driver
        self.plugin = self.openstack_driver.plugin
        self.api_helper = MapNeutronToKemp(self.plugin)

    @property
    def load_balancer(self):
        return LoadBalancerManager(self, self.openstack_driver.load_balancer)

    @property
    def listener(self):
        return ListenerManager(self, self.openstack_driver.listener)

    @property
    def pool(self):
        return PoolManager(self, self.openstack_driver.pool)

    @property
    def member(self):
        return MemberManager(self, self.openstack_driver.member)

    @property
    def health_monitor(self):
        return HealthMonitorManager(self, self.openstack_driver.health_monitor)

    def default_checker_settings(self):
        """Return default health check parameters."""
        return {
            'retryinterval': self.check_interval,
            'timeout': self.connect_timeout,
            'retrycount': self.retry_count,
        }



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
import logging as log
import unittest

import mock

from kemptech_openstack_lbaas import driver as kemptech
from kemptech_openstack_lbaas import client

LOG = log.getLogger(__name__)

CONF = mock.Mock()
CONF.lm_address = '10.0.0.1'
CONF.lm_username = 'bal'
CONF.lm_password = '2fourall'
CONF.check_interval = 9
CONF.connect_timeout = 4
CONF.retry_count = 2


class TestKempLoadMasterDriver(unittest.TestCase):
    """Test suite to test methods of KempLoadMasterDriver."""
    def setUp(self):

        self.plugin_mock = mock.Mock()
        self.driver = kemptech.KempLoadMasterDriver(self.plugin_mock)


class TestLoadBalancerManager(unittest.TestCase):
    """Test suite to test methods of LoadBalancerManager."""
    def setUp(self):
        self.client_mock = mock.Mock(spec=client.KempClient)
        self.api_helper_mock = mock.Mock(spec=kemptech.MapNeutronToKemp)
        self.lb = kemptech.LoadBalancerManager()

    def test_create(self, model):
        self.lb.create(model)

    def test_update(self, old_model, model):
        self.lb.update(old_model, model)

    def test_delete(self, model):
        self.lb.delete(model)


class TestListenerManager(unittest.TestCase):
    """Test suite to test methods of ListenerManager."""
    def setUp(self):
        super(TestListenerManager, self).__init__()
        self.client_mock = mock.Mock(spec=client.KempClient)
        self.api_helper_mock = mock.Mock(spec=kemptech.MapNeutronToKemp)
        self.listener = kemptech.ListenerManager()

    def test_create(self, model):
        self.lb.create(model)

    def test_update(self, old_model, model):
        self.lb.update(old_model, model)

    def test_delete(self, model):
        self.lb.delete(model)

class TestPoolManager(unittest.TestCase):
    """Test suite to test methods of PoolManager."""
    def setUp(self):
        super(TestPoolManager, self).__init__()
        self.client_mock = mock.Mock(spec=client.KempClient)
        self.api_helper_mock = mock.Mock(spec=kemptech.MapNeutronToKemp)
        self.pool = kemptech.PoolManager()

    def test_create(self, model):
        self.pool.create(model)

    def test_update(self, old_model, model):
        self.pool.update(old_model, model)

    def test_delete(self, model):
        self.pool.delete(model)


class TestMemberManager(unittest.TestCase):
    """Test suite to test methods of MemberManager."""
    def setUp(self):
        super(TestMemberManager, self).__init__()
        self.client_mock = mock.Mock(spec=client.KempClient)
        self.api_helper_mock = mock.Mock(spec=kemptech.MapNeutronToKemp)
        self.member = kemptech.MemberManager()

    def test_create(self, model):
        self.member.create(model)

    def test_update(self, old_model, model):
        self.member.update(old_model, model)

    def test_delete(self, model):
        self.member.delete(model)


class TestHealthMonitorManager(unittest.TestCase):
    """Test suite to test methods of KempLoadMasterDriver."""
    def setUp(self):
        super(TestHealthMonitorManager, self).__init__()
        self.client_mock = mock.Mock(spec=client.KempClient)
        self.api_helper_mock = mock.Mock(spec=kemptech.MapNeutronToKemp)
        self.hm = kemptech.HealthMonitorManager()

    def test_create(self, model):
        self.hm.create(model)

    def test_update(self, old_model, model):
        self.hm.update(old_model, model)

    def test_delete(self, model):
        self.hm.delete(model)




#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import abc
from kemptech_openstack_lbaas import constants as kemp_consts
from neutron.common import exceptions
from oslo_log import log
import requests
import six

LOG = log.getLogger(__name__)


class KempClient(object):

    def __init__(self, address, username, password):
        if (address is None or
            username is None or
            password is None):
            raise KempClientRequestError(msg="Missing URL credentials.")
        else:
            self.loadbalancer = ("https://%s:%s@%s/access/"
                    % (username, password, address))
            self.virtual_services = []
            self.real_servers = []

    def create_virtual_service(self, params):
        virtual_service = VirtualService(self.loadbalancer, params)
        virtual_service.create()
        self.virtual_services.append(virtual_service)

    def update_virtual_service(self, old_params, params, batch_update=False):
        old_virtual_service = VirtualService(self.loadbalancer, old_params)
        virtual_service = VirtualService(self.loadbalancer, params)
        if batch_update:
            pass
        else:
            for service in self.virtual_services:
                if service.id == old_virtual_service.id:
                    service.update(virtual_service.to_dict())
        self.virtual_services.remove(old_virtual_service)
        self.virtual_services.append(virtual_service)

    def delete_virtual_service(self, params):
        virtual_service = VirtualService(self.loadbalancer, params)
        for service in self.virtual_services:
            if service.id == virtual_service.id:
                service.delete()

    def delete_virtual_services(self):
        for virtual_service in self.virtual_services:
            virtual_service.delete()

    def create_real_server(self, params):
        """Attach a real server to a virtual service."""
        real_server = RealServer(self.loadbalancer, params)
        real_server.create()
        self.real_servers.append(real_server)

    def update_real_server(self, old_params, params):
        """Update a real server's parameters on a virtual service."""
        old_real_server = RealServer(self.loadbalancer, old_params)
        real_server = RealServer(self.loadbalancer, params)
        for server in self.real_servers:
            if server.id == old_real_server.id:
                server.update(real_server.to_dict())
        self.real_servers.remove(old_real_server)
        self.real_servers.append(real_server)

    def delete_real_server(self, params):
        """Delete a real server from a virtual service."""
        real_server = RealServer(self.loadbalancer, params)
        for server in self.real_servers:
            if server.id == real_server.id:
                server.delete()

    def update_health_check(self, check_params):
        """Update health check parameters for virtual services."""
        # pop the checker params, call update vs
        # create a health and a vs object
        checkers = {}
        for checker in kemp_consts.CHECKER_OPTS:
            checkers.update(check_params.popitem(checker))
        health_monitor = HealthMonitor(self.loadbalancer, checkers)
        health_monitor.update()
        virtual_service = VirtualService(self.loadbalancer, check_params)
        for service in self.virtual_services:
            if service.id == virtual_service.id:
                service.update(virtual_service.to_dict())


class BaseKempModel(object):
    """A class to build objects based on KEMP RESTful API.

    Subclasses built from this class need to name their parameters
    the same as their RESTful API counterpart in order for this
    class to work.
    """

    API_NAME = ""  # Subclasses must define this.

    def __init__(self, loadbalancer, parameters):
        self.loadbalancer = loadbalancer
        for api_key, api_value in parameters:
            self.__dict__[api_key] = api_value
        if not self.exists:
            self._get_request('add' + self.API_NAME, self.id)

    def create(self):
        if not self.exists:
            self._get_request('add' + self.API_NAME, self.id)

    def update(self, obj):
        if self.exists:
            # Remove the ID parameters and update the attributes.
            for model_id in self.id:
                if model_id in obj.__dict__:
                    obj.__dict__.pop(model_id)
            self.__dict__.update(obj.__dict__)
            self._get_request('mod' + self.API_NAME, self.to_dict())

    def delete(self):
        if self.exists:
            self._get_request('del' + self.API_NAME, self.id)

    @property
    def exists(self):
        return self._get_request('show' + self.API_NAME, self.id) < 300

    @abc.abstractproperty
    def id(self):
        """Must return a dict with unique ID parameters for KEMP API."""
        pass

    @id.setter
    @abc.abstractmethod
    def id(self, value):
        pass

    def to_dict(self):
        """Return a dictionary containing attributes of class.

        Ignore attributes that are set to None or are not a string or int;
        also ignore id as it is not an API thing.
        """
        attributes = {}
        for attr in self.__dict__:
            if (self.__dict__[attr] is not None or
                not self.__dict__[attr] == 'id' or
                not isinstance(self.__dict__[attr], six.string_types) or
                not isinstance(self.__dict__[attr], (int, long))):
                attributes[attr] = self.__dict__[attr]
        return attributes

    def _get_request(self, cmd, params):
        """Perform a HTTP GET.
        Arguments:
            cmd - The command to run.
            params - Dict containing parameters.
        Returns:
            The status code of request
            The response text body.
        """
        cmd_url = self.loadbalancer + cmd + "?"
        LOG.debug("cmd_url: %s" % cmd_url)
        LOG.debug("params: %s" % repr(params))

        try:
            response = requests.request('GET', cmd_url, params=params,
                                        verify=False)
        except requests.exceptions.ConnectionError:
            LOG.error(_LE("A connection error occurred to %s"),
                      self.loadbalancer)
            raise KempClientRequestError(response.status_code, response.text)
        except requests.exceptions.URLRequired:
            LOG.error(_LE("%s is not a valid URL to make a request with."),
                      cmd_url)
            raise KempClientRequestError(response.status_code, response.text)
        except requests.exceptions.TooManyRedirects:
            LOG.error(_LE("Too many redirects with requst to %s"), cmd_url)
            raise KempClientRequestError(response.status_code, response.text)
        except requests.exceptions.Timeout:
            LOG.error(_LE("A connection error occurred to %s"),
                      self.loadbalancer)
            raise KempClientRequestError(response.status_code, response.text)
        except requests.exceptions.RequestException:
            LOG.error(_LE("An uknown error occurred with request to %s"),
                      cmd_url)
            raise KempClientRequestError(response.status_code, response.text)
        return response.status_code, response.text


class VirtualService(BaseKempModel):

    API_NAME = 'vs'

    def __init__(self, loadbalancer, parameters):
        self.id = parameters
        super(VirtualService, self).__init__(loadbalancer, parameters)

    @property
    def id(self):
        return {
            'vs': self.vs,
            'port': self.port,
            'prot': self.prot,
        }

    @id.setter
    def id(self, value):
        self.vs = value['vs']
        self.port = value['port']
        self.prot = value['prot']
        self.rs = value['rs']
        self.rsport = value['rsport']

    def update(self, virtual_service):
        if self.vsport is not None:
            if super(VirtualService, self).update() < 300:
                self.port = self.vsport
                self.vsport = None
        else:
            super(VirtualService, self).update(virtual_service)


class RealServer(BaseKempModel):

    API_NAME = 'rs'

    def __init__(self, loadbalancer, parameters):
        self.id = parameters
        super(RealServer, self).__init__(loadbalancer, parameters)

    @property
    def id(self):
        return {
            'vs': self.vs,
            'port': self.port,
            'prot': self.prot,
            'rs': self.rs,
            'rsport': self.rsport,
        }

    @id.setter
    def id(self, value):
        self.vs = value['vs']
        self.rs = value['rs']
        self.port = value['port']
        self.prot = value['prot']
        self.rsport = value['rsport']


class HealthMonitor(BaseKempModel):

    API_NAME = 'health'

    def __init__(self, parameters):
        self.id = parameters
        super(HealthMonitor, self).__init__(parameters)

    @property
    def id(self):
        """HealthMonitor's do not have a unique ID as they are global"""
        return None

    @id.setter
    def id(self, value):
        self.vs = value['vs']
        self.rs = value['rs']
        self.port = value['port']
        self.prot = value['prot']
        self.rsport = value['rsport']


class KempClientRequestError(exceptions.NeutronException):
    """Raised if HTTP request has failed."""

    def __init__(self, code=None, msg=None):
        if msg is None:
            if code == 400:
                msg = "Mandatory parameter missing from request."
            elif code == 401:
                msg = "Username or password is missing or is incorrect."
            elif code == 403:
                msg = "Incorrect permissions."
            elif code == 404:
                msg = "Not found."
            elif code == 405:
                msg = "Unknown command."
            else:
                msg = "An unknown error has occurred."
        self.message = _("KEMP Client Error: %(code)s; %(msg)s") % (code, msg)
        super(KempClientRequestError, self).__init__()



#!/usr/bin/env python

from setuptools import setup
from setuptools import find_packages

setup(name="kemptech-openstack-lbaas",
      version="0.9.5.4",
      packages=find_packages(),
      author="KEMP Technologies",
      author_email="smcgough@kemptechnologies.com",
      maintainer="Shane McGough",
      maintainer_email="smcgough@kemptechnologies.com",
      description="KEMP Technologies OpenStack LBaaS Driver V2",
      long_description=open('README.rst').read(),
      license="Apache",
      keywords="kemptech kemp technologies lbaas load balancer driver",
      url="http://pypi.python.org/pypi/kemptech-openstack-lbaas/",
      classifiers=[
          'License :: OSI Approved :: Apache Software License',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Internet',
      ],
      install_requires=['requests>=2.3.0',
                        'mock',
                        'six'])
